/*= driver MTX329x ========================================================*/
/* Original Release: 01/06/2015                                            */
/* By: O.L.                                                                */
/* Originally written in C                                                 */
/*=========================================================================*/


#include "mtx329x.h"
#include "rs232.h"
#include "formatio.h"
#include "utility.h"
#include "ansi_c.h"

#define TEMPO			0.1

#define MTX329x_VDC		0
#define MTX329x_VAC		1
#define MTX329x_VACDC	2
#define MTX329x_VLOWZ	3
#define MTX329x_ADC		4
#define MTX329x_AAC		5
#define MTX329x_AACDC	6
#define MTX329x_HZ 		7
#define MTX329x_F  		8
#define MTX329x_OHM		9
#define MTX329x_VD 		10
#define MTX329x_DEGC 	11
#define MTX329x_DEGF 	12



/*= STATIC VARIABLES ======================================================*/
static int mtx329x_model;
char *mtx329x_Function[4] = {"DC", "AC", "ACDC", "VLOWZ"};
char mtx329x_smeasure[9];
int mtx329x_typeMesure;
int mtx329x_unit;


/*= UTILITY ROUTINES ======================================================*/
int mtx329x_invalid_integer_range (int val, int min, int max, int err_code);
int mtx329x_sendData (int portCom, char *commande);
int mtx329x_readData (int portCom, char *buff);
double mtx329x_CoeffMesure (char mult);


/*=========================================================================*/
/* Cette fonction ouvre le port srie , interroge l'appareil connect au   */
/* PC, reset l'appareil.                                                   */
/*=========================================================================*/
int _VI_FUNC mtx329x_init (int portCom, int IDQuery, int resetDevice)
{
	char buff[50];
	int mtx329x_err = 0;

    if (mtx329x_invalid_integer_range (portCom, 1, 255, ERROR_PARAMETER1) != 0)
        return mtx329x_err;
    if (mtx329x_invalid_integer_range (IDQuery, 0, 1, ERROR_PARAMETER2) != 0)
        return mtx329x_err;
    if (mtx329x_invalid_integer_range (resetDevice, 0, 1, ERROR_PARAMETER3) != 0)
        return mtx329x_err;

	// open serial port
	OpenComConfig (portCom, "", 38400, 0, 8, 1, 512, 512);
    
    if (rs232err == 0)
    	{
	    SetComTime (portCom, 0.1);
	    SetXMode (portCom, 0);
		SetCTSMode (portCom, LWRS_HWHANDSHAKE_OFF);
		
		if (IDQuery)
			{
			mtx329x_err = mtx329x_sendData (portCom, "\r*IDN?\r");
			mtx329x_err = mtx329x_readData (portCom, buff);
			
			// passer de ModBus  SCPI : 0x8002
			
			if (FindPattern (buff, 0, -1, "MTX 329", 0, 0) == -1)
				{
				mtx329x_err = -1;
				CloseCom (portCom);
				}
			else
				{
				Scan (buff, "MTX %i", &mtx329x_model);
				}
			}

		if (resetDevice)
			mtx329x_err = mtx329x_sendData (portCom, "*RST\r");

		// effacer les erreurs
		mtx329x_sendData (portCom, "*CLS\r");
		}
    else
		mtx329x_err = rs232err;
		
	return mtx329x_err;
}


/*=========================================================================*/
/* Cette fonction configure les mesures de tension de l'appareil	       */
/*=========================================================================*/
int _VI_FUNC mtx329x_voltage (int portCom, int function, int range)
{
	double Range[6] = {0, 0.05, 0.5, 5, 50, 500};
	char message[200];
	int mtx329x_err = 0;

    if (mtx329x_invalid_integer_range (function, 0, 3, ERROR_PARAMETER1) != 0)
        return mtx329x_err;
    if (mtx329x_invalid_integer_range (range, 0, 5, ERROR_PARAMETER2) != 0)
        return mtx329x_err;

	// send command
	if (function == 3)	// function VlowZ
	{
		mtx329x_err = mtx329x_sendData (portCom, "FUNC \"LOWZ\"\r");
		mtx329x_typeMesure = MTX329x_VLOWZ;
	}
	else
		mtx329x_err = mtx329x_sendData (portCom, "FUNC \"VOLT\"\r");
	
	if (mtx329x_err == 0)
	{
		// Coupling
		if (function < 3)
		{
			Fmt (message, "INP:COUP %s\r", mtx329x_Function[function]);
			mtx329x_err = mtx329x_sendData (portCom, message);
			mtx329x_typeMesure = MTX329x_VDC + function;
		}
	
		// AutoRange On/Off
		Fmt (message, "RANG:AUTO %i\r", range == 0);
		mtx329x_err = mtx329x_sendData (portCom, message);

		// Ranging
		if (range != 0)
		{
			Fmt (message, "RANG %f\r", Range[range]);
			mtx329x_err = mtx329x_sendData (portCom, message);
		}
	}

	return mtx329x_err;
}

/*=========================================================================*/
/* Cette fonction configure les mesures de courant de l'appareil	       */
/*=========================================================================*/
int _VI_FUNC mtx329x_current (int portCom, int function, int range)
{
	double Range[7] = {0, 0.0005, 0.005, 0.05, 0.5, 5.0, 15.0};
	char message[200];
	int mtx329x_err = 0;

    if (mtx329x_invalid_integer_range (function, 0, 2, ERROR_PARAMETER1) != 0)
        return mtx329x_err;
    if (mtx329x_invalid_integer_range (range, 0, 6, ERROR_PARAMETER2) != 0)
        return mtx329x_err;

	// send command
	mtx329x_err = mtx329x_sendData (portCom, "FUNC \"CURR\"\r");
	
	if (mtx329x_err == 0)
	{
		// Coupling
		Fmt (message, "INP:COUP %s\r", mtx329x_Function[function]);
		mtx329x_err = mtx329x_sendData (portCom, message);
		mtx329x_typeMesure = MTX329x_ADC + function;
	
		// AutoRange On/Off
		Fmt (message, "RANG:AUTO %i\r", range == 0);
		mtx329x_err = mtx329x_sendData (portCom, message);

		// Ranging
		if (range != 0)
		{
			Fmt (message, "RANG %f\r", Range[range]);
			mtx329x_err = mtx329x_sendData (portCom, message);
		}
	}
	
	return mtx329x_err;
}

/*=========================================================================*/
/* Cette fonction configure les mesures de frquence de l'appareil	       */
/*=========================================================================*/
int _VI_FUNC mtx329x_frequency (int portCom, int mode, int range)
{
	char *Range[2] = {"INF200KHZ", "SUP200KHZ"};
	double RangeV[6] = {0, 0.05, 0.5, 5, 50, 500};
	char message[200];
	int mtx329x_err = 0;

    if (mtx329x_invalid_integer_range (mode, 0, 1, ERROR_PARAMETER1) != 0)
        return mtx329x_err;
    if (mtx329x_invalid_integer_range (range, 1, 5, ERROR_PARAMETER2) != 0)
        return mtx329x_err;

	// send command
	mtx329x_err = mtx329x_sendData (portCom, "FUNC \"FREQ\"\r");
	mtx329x_typeMesure = MTX329x_HZ;
	
	// Frequence mode
	Fmt (message, "FREQ:MOD %s\r", Range[mode]);
	mtx329x_err = mtx329x_sendData (portCom, message);
		
	// Ranging volt
	Fmt (message, "FREQ:THR:VOLT:RANG %f\r", RangeV[range]);
	mtx329x_err = mtx329x_sendData (portCom, message);
	
	return mtx329x_err;
}

/*=========================================================================*/
/* Cette fonction configure les mesures de resistance et test diode de     */
/* l'appareil														       */
/*=========================================================================*/
int _VI_FUNC mtx329x_resistance (int portCom, int measure, int range)
{
	char *Function[5] = {"RES", "CONT", "100", "DIOD", "DIODEZ"};
	double Range[7] = {0, 500, 5000, 50000, 500000, 5000000, 50000000};
	char message[200];
	int mtx329x_err = 0;

    if (mtx329x_invalid_integer_range (measure, 0, 4, ERROR_PARAMETER1) != 0)
        return mtx329x_err;
    if (mtx329x_invalid_integer_range (range, 0, 6, ERROR_PARAMETER2) != 0)
        return mtx329x_err;

	// send command
	Fmt (message, "FUNC \"%s\"\r", Function[measure]);
	mtx329x_err = mtx329x_sendData (portCom, message);
	
	if (mtx329x_err == 0)
	{
		if (measure > 2)
			mtx329x_typeMesure = MTX329x_VD;
		else
			mtx329x_typeMesure = MTX329x_OHM;

		if (measure == 0)
		{
			// AutoRange On/Off
			Fmt (message, "RANG:AUTO %i\r", range == 0);
			mtx329x_err = mtx329x_sendData (portCom, message);

			// Ranging
			if (range != 0)
			{
				Fmt (message, "RANG %f\r", Range[range]);
				mtx329x_err = mtx329x_sendData (portCom, message);
			}
		}
	}
		
	return mtx329x_err;
}

/*=========================================================================*/
/* Cette fonction configure les mesures de capacite de l'appareil		   */
/*=========================================================================*/
int _VI_FUNC mtx329x_capacitance (int portCom, int range)
{
	double Range[9] = {0, 500e-12, 5e-9, 50e-9, 500e-9, 5e-6, 50e-6, 500e-6, 5e-3};
	char message[200];
	int mtx329x_err = 0;

    if (mtx329x_invalid_integer_range (range, 0, 8, ERROR_PARAMETER1) != 0)
        return mtx329x_err;

	// send command
	mtx329x_err = mtx329x_sendData (portCom, "FUNC \"CAPA\"\r");
	mtx329x_typeMesure = MTX329x_F;
	
	if (mtx329x_err == 0)
	{
		// AutoRange On/Off
		Fmt (message, "RANG:AUTO %i\r", range == 0);
		mtx329x_err = mtx329x_sendData (portCom, message);

		// Ranging
		if (range != 0)
		{
			Fmt (message, "RANG %f\r", Range[range]);
			mtx329x_err = mtx329x_sendData (portCom, message);
		}
	}
		
	return mtx329x_err;
}

/*=========================================================================*/
/* Cette fonction configure les mesures de capacite de l'appareil		   */
/*=========================================================================*/
int _VI_FUNC mtx329x_temperature (int portCom, int unit, int sensor)
{
	char *Unit[3] = {"C", "F", "K"};
	char *Sensor[4] = {"PT100", "PT1000", "TCJ", "TCK"};
	char message[200];
	int mtx329x_err = 0;

    if (mtx329x_invalid_integer_range (unit, 0, 2, ERROR_PARAMETER1) != 0)
        return mtx329x_err;
    if (mtx329x_invalid_integer_range (sensor, 0, 3, ERROR_PARAMETER2) != 0)
        return mtx329x_err;

	// send command
	mtx329x_err = mtx329x_sendData (portCom, "FUNC \"TEMP\"\r");
	
	if (mtx329x_err == 0)
	{
		Fmt (message, "UNIT:TEMP %s\r", Unit[unit]);
		mtx329x_err = mtx329x_sendData (portCom, message);
		mtx329x_typeMesure = MTX329x_DEGC + unit;
		
		// SensorRanging
		Fmt (message, "TEMP:TRAN %s\r", Sensor[sensor]);
		mtx329x_err = mtx329x_sendData (portCom, message);
	}
		
	return mtx329x_err;
}

/*=========================================================================*/
/* Cette fonction configure le filtre de l'appareil			   			   */
/*=========================================================================*/
int _VI_FUNC mtx329x_filter (int portCom, int filter)
{
	char message[200];
	int mtx329x_err = 0;

    if (mtx329x_invalid_integer_range (filter, 0, 1, ERROR_PARAMETER1) != 0)
        return mtx329x_err;

	// send command
	Fmt (message, "FILT %i\r", filter);
	mtx329x_err = mtx329x_sendData (portCom, message);

	return mtx329x_err;
}

/*=========================================================================*/
/* Cette fonction configure l'impedance d'entre de l'appareil 			   */
/*=========================================================================*/
int _VI_FUNC mtx329x_impedance (int portCom, int impedance)
{
	char message[200];
	int mtx329x_err = 0;
	double valImpedance[2] = {1e7, 1e9};

    if (mtx329x_invalid_integer_range (impedance, 0, 1, ERROR_PARAMETER1) != 0)
        return mtx329x_err;

	// send command
	Fmt (message, "INP:IMP %f\r", valImpedance[impedance]);
	mtx329x_err = mtx329x_sendData (portCom, message);

	return mtx329x_err;
}

/*=========================================================================*/
/* Cette fonction configure le filtre de l'appareil			   			   */
/*=========================================================================*/
int _VI_FUNC mtx329x_dBmRef (int portCom, int dbm_Ref)
{
	char message[200];
	int mtx329x_err = 0;

    if (mtx329x_invalid_integer_range (dbm_Ref, 1, 10000, ERROR_PARAMETER1) != 0)
        return mtx329x_err;

	// send command
	Fmt (message, "MENU:DBM:IMP %i\r", dbm_Ref);
	mtx329x_err = mtx329x_sendData (portCom, message);

	return mtx329x_err;
}

/*=========================================================================*/
/* Cette fonction configure le filtre de l'appareil			   			   */
/*=========================================================================*/
int _VI_FUNC mtx329x_WattRef (int portCom, int watt_Ref)
{
	char message[200];
	int mtx329x_err = 0;

    if (mtx329x_invalid_integer_range (watt_Ref, 1, 10000, ERROR_PARAMETER1) != 0)
        return mtx329x_err;

	// send command
	Fmt (message, "MENU:WATT:IMP %i\r", watt_Ref);
	mtx329x_err = mtx329x_sendData (portCom, message);

	return mtx329x_err;
}

/*=========================================================================*/
/* Cette fonction lit la mesure en cours							   	   */
/*=========================================================================*/
int _VI_FUNC mtx329x_measure (int portCom, double *measure1)
{
	static unsigned char message[200];
	int mtx329x_err = 0;
	
	memset (message, 0, 200);
	mtx329x_err = mtx329x_sendData (portCom, "MEAS?\r");
	mtx329x_err = mtx329x_readData (portCom, message);
	
	// test overload "OL"
	if (FindPattern (message, 0, 8, "OL", 0, 0) != -1 || 
		FindPattern (message, 0, 8, "----", 0, 0) != -1)
	{
    	mtx329x_err = 303;
		Fmt (mtx329x_smeasure, "  XXXXX");
		
		if (FindPattern (message, 0, 8, "----", 0, 0) == -1)
		{
			if (message[0] == '-')
			{
		    	mtx329x_err = 305;
				mtx329x_smeasure[0] = '-';
			}
		}
	}
	else
	{	// recuperer la valeur numrique
		Scan (message, "%f", measure1);
	}
	
	return mtx329x_err;
}

/*=========================================================================*/
/* Cette function ferme le port srie                                      */
/*                                                                         */
/*=========================================================================*/
int _VI_FUNC mtx329x_error (int portCom, char *message)
{
	int index;
	int mtx329x_err = 0;
	
	mtx329x_err = mtx329x_sendData (portCom, "SYST:ERR?\r");
	mtx329x_err = mtx329x_readData (portCom, message);
	index = FindPattern (message, 0, -1, "\n", 0, 0);
	if (index != -1)
		*(message + index) = 0x0;
	
	return mtx329x_err;
}


/*=========================================================================*/
/* Cette function ferme le port srie                                      */
/*                                                                         */
/*=========================================================================*/
int _VI_FUNC mtx329x_close (int portCom)
{
	int mtx329x_err = 0;

    mtx329x_err = CloseCom (portCom);
	
	return mtx329x_err;
}


/*=========================================================================*/
/* cette fonction crit dans un buffer de data la chaine de commande       */
/*  envoyer                                                               */
/*=========================================================================*/
int mtx329x_sendData (int portCom, char *commande)
{
	int mtx329x_err = 0;
	
	FlushInQ (portCom);
	FlushOutQ (portCom);
    ComWrt (portCom, commande, strlen (commande));
    Delay (TEMPO);
    
    if (rs232err != 0)
		mtx329x_err = rs232err;
		
	return mtx329x_err;
}


/*=========================================================================*/
/* cette fonction lit dans un buffer la chaine de commande renvoye        */
/*                                                                         */
/*=========================================================================*/
int mtx329x_readData (int portCom, char *buff)
{
	int mtx329x_err = 0;

    ComRdTerm (portCom, buff, 50, 0x0D);

    if (rs232err != 0)
		mtx329x_err = rs232err;
		
	return mtx329x_err;
}

/*=========================================================================*/
/* cette fonction applique le multiplicateur  la mesure			       */
/*                                                                         */
/*=========================================================================*/
double mtx329x_CoeffMesure (char mult)
	{
	double measure = 1.0;
	
	switch (mult)
		{
		case 'n' : measure = 1e-9; break;
		case '' : measure = 1e-6; break;
		case 'm' : measure = 1e-3; break;
		case 'k' : measure = 1e+3; break;
		case 'M' : measure = 1e+6; break;
		case 'G' : measure = 1e+9; break;
		}
		
	return measure;
	}

/*=========================================================================*/
/* Function: Invalid Integer Range                                         */
/* Purpose:  This function checks an integer to see if it lies between a   */
/*           minimum and maximum value.  If the value is out of range, set */
/*           the global error variable to the value err_code.  If the      */
/*           value is OK, error = 0.                                       */
/*=========================================================================*/
int mtx329x_invalid_integer_range (int val, int min, int max, int err_code)
{
	int mtx329x_err = 0;
	
	if ((val < min) || (val > max))
    {
    	mtx329x_err = err_code;
    	return mtx329x_err;
    }
  	return 0;
}

